import json
import multiprocessing as mp
import sys
sys.path.insert(0, "/home/gridsan/hwpang/RMG_shared/Projects/Hao-Wei-Oscar/AEC_BAC_part_1/RMG-Py")
sys.path.insert(0, "/home/gridsan/oscarwu/RMG_shared/Projects/Hao-Wei-Oscar/AEC_BAC_part_1/RMG-Py")
from datetime import datetime
from tqdm import tqdm
from joblib import Parallel, delayed

from arkane.encorr.isodesmic import ErrorCancelingScheme
from arkane.encorr.reference import ReferenceDatabase
from arkane.modelchem import LevelOfTheory

JOB_ID = int(sys.argv[1])
N_JOBS = int(sys.argv[2])
# N_TASKS = 80
CLASSES = ['rc2', 'rc3', 'rc4']
LOT_DICT = {
    'B2PLYPD3': LevelOfTheory(method='B2PLYPD3-2023',
                              basis='def2tzvp',
                              software='gaussian',
                              ),

    'b3lyp': LevelOfTheory(method='b3lyp-2023',
                           basis='def2tzvp',
                           software='gaussian',
                            ),
    
    'b97d3': LevelOfTheory(method='b97d3-2023',
                           basis='def2tzvp',
                           software='gaussian',
                           ),
    
    'BP86': LevelOfTheory(method='BP86-2023',
                          basis='def2tzvp',
                          software='gaussian',
                          ),
    
    'CAM-B3LYP': LevelOfTheory(method='CAM-B3LYP-2023',
                               basis='def2tzvp',
                               software='gaussian',
                            ),

    'cbs-qb3': LevelOfTheory(method='cbsqb3-2023',
                             software='gaussian'),

    'dlpno-ccsdt-def2-TZVP': LevelOfTheory(method='dlpno-ccsd(t)-2023', basis='def2-tzvp', software='orca'),

    'dlpno-ccsdt-f12-ccpvdz': LevelOfTheory(method='dlpno-ccsd(t)-f12-2023', basis='cc-pvdz-f12', software='orca'),

    'dlpno-ccsdt-f12-ccpvtz': LevelOfTheory(method='dlpno-ccsd(t)-f12-2023', basis='cc-pvtz-f12', software='orca'),

    'g4': LevelOfTheory(method='g4-2023',
                        software='gaussian'),

    'M062X': LevelOfTheory(method='M062X-2023',  
                           basis='def2tzvp',
                           software='gaussian',
                           ),

    'PBE1PBE': LevelOfTheory(method='PBE1PBE-2023',  
                           basis='def2tzvp',
                           software='gaussian',
                           ),

    'TPSSTPSS': LevelOfTheory(method='TPSSTPSS-2023',
                              basis='def2tzvp',
                              software='gaussian',
                              ),

    'wb97mv': LevelOfTheory(method='wb97mv-2023',
                            basis='def2tzvpd',
                            software='qchem',
                            ),

    'wb97xd': LevelOfTheory(method='wb97xd-2023',
                              basis='def2tzvp',
                              software='gaussian',
                              ),              
}

all_jobs = [(rc_class, lot_label) for rc_class in CLASSES for lot_label in LOT_DICT.keys()]

RC_CLASS, LOT_LABEL = all_jobs[JOB_ID]
CONSTRAINT = (False, False, False)

database = ReferenceDatabase()
database.load()

lot = LOT_DICT[LOT_LABEL]

ref_set = database.reference_sets['main']
ref_mapping = {}
for spcs in ref_set:
    try:
        molecule = spcs.to_error_canceling_spcs(lot).molecule
        # if LOT_LABEL in ('B', 'C', 'D'):
        #     if molecule.get_net_charge() < 0:
        #         continue
        #     if 'F' in molecule.get_element_count().keys():
        #         continue
        ref_mapping[spcs.index] = spcs.to_error_canceling_spcs(lot)
    except KeyError:
        pass


def process(tup):
    indx, spc = tup
    error_spcs = [value for key, value in ref_mapping.items() if key != indx]
    scheme = ErrorCancelingScheme(target=spc,
                                  reference_set=error_spcs,
                                  isodesmic_class=RC_CLASS,
                                  conserve_ring_size=CONSTRAINT[0],
                                  limit_charges=CONSTRAINT[1],
                                  limit_scope=CONSTRAINT[2],
                                  )

    h298, rxns = scheme.calculate_target_enthalpy(n_reactions_max=5)
    lit_h298 = spc.high_level_hf298
    if h298 is not None:
        data = {'Index': indx,
                'CalcHf298': h298.value_si,
                'LitHf298': lit_h298.value_si,
                'Error': (h298.value_si - lit_h298.value_si)/4184.0,
                'Reactions': [rxn.__repr__() for rxn in rxns],
                }
    else:
        data = {'Index': indx,
                'CalcHf298': h298,
                'LitHf298': lit_h298.value_si,
                'Error': None,
                'Reactions': [rxn.__repr__() for rxn in rxns],
                }
    return data

datas = Parallel(n_jobs=N_JOBS)(delayed(process)(tup) for tup in tqdm(list(ref_mapping.items())))
full_data = {indx: data for indx, data in zip(ref_mapping.keys(), datas)}

# full_data = dict()
# for indx, spc in tqdm(ref_mapping.items()):
#     tup = indx, spc
#     data = process(tup)
#     full_data[indx] = data

# with mp.Manager() as manager:
#     full_data = manager.dict()

#     pool = mp.Pool(N_TASKS)
#     results = pool.map(process, list(ref_mapping.items()))
#     pool.close()
#     pool.join()

#     full_data = dict(full_data)

if any(CONSTRAINT):
    constraint_str = ''
    if CONSTRAINT[0]:
        constraint_str += 'limit_charges'
    if CONSTRAINT[1]:
        constraint_str += '_and_scope'
else:
    constraint_str = 'full'

# with open(f'./full_data_pickles/M2_{LOT_LABEL}_{RC_CLASS}_{constraint_str}.pkl', 'wb') as f:
#     pickle.dump(full_data, f)

with open(f'isodesmic/M0_{LOT_LABEL}_{RC_CLASS}_{constraint_str}.json', 'w') as f:
    json.dump(full_data, f)
